      subroutine Qromb (Funcs,Aout,Bout, SS,EPS, Switch) 

c
c  Romberg's method of numerical quadrature, from Numerical Recipes.
c  Given a function FUNC and limits Aout and Bout, its definite 
c  integral SS is returned to fractional accuracy EPS. 
c
c Adjusted version built from routines in seidm.f by SC !! 

      real Aout,Bout, SS, EPS 
      EXTERNAL  Funcs
      integer   Switch 
      
      parameter( jmax=50, jmaxp=jmax+1, k=5, km=k-1 ) 
c     integer, parameter :: jmax=50, jmaxp=jmax+1, k=2, km=k-1 
c     k=2 means Simpson's ! 

      dimension s(jmaxp),h(jmaxp)

      integer i,j, IT, TNM, jj    

      real  s0, X, DEL, SUM, DSS, FUNC, INTGRND, i1, i2    

c  Compute the jth stage of refinement of an extended trapezoidal
c  rule integration between A and B (from Numerical Recipes' TRAPZD).

      h(1) = 1.0 

      do j = 1,jmax 
   
         IF (j.EQ.1) THEN 
            i1 = Funcs(Aout, Switch)
            i2 = Funcs(Bout, Switch)
            s0 = 0.5*(Bout-Aout)*(i1+i2)
            IT = 1
         ELSE 
            TNM = IT 
            DEL = (Bout-Aout)/float(TNM) 
            X   = Aout + 0.5*DEL 
            SUM = 0.0 
            do jj = 1,IT 
               SUM = SUM + Funcs(X, Switch)
               X = X + DEL 
            enddo
            s0 = 0.5*(s0+(Bout-Aout)*SUM/TNM)
            IT = 2*IT 
         ENDIF
         s(j) = s0 
c Use Neville's algorithm to extrapolate to zero -- routine polint, 
c from Num rec, and SC.  
         IF (j.GE.k) THEN 
            call Polint(h(j-km),s(j-km),k,SS,DSS)
            IF (DSS.le.(EPS*abs(SS))) RETURN 
         ENDIF
c
         s(j+1) = s(j) 
         h(j+1) = 0.25*h(j) 
c
      enddo
c
      write(*,*) ' TOO MANY steps in ROMBERG integration! '
c
      end subroutine QRomb

      subroutine Polint (XA,YA,n,Y,DY)

c
c  Given arrays XA and YA, each of length N, and given a value X=0, this
c  routine (from Numerical Recipes) returns a value Y, and an error
c  estimate DY, that is interpolated or extrapolated from the polynomial
c  defined by YA(XA).   NOTE:  CUSTOMIZED FOR USE WITH ROMBERG, ABOVE!
c
c---------------------------------------------------------------------------
c
c NOTE: Adapted by JS from SC, seimd.f -- BUT, here k=n 
      
      PARAMETER (NMAX=10 )
      integer n 
      dimension  XA(N),YA(N) 
      real  Y, DY  
      dimension  C(NMAX),D(NMAX)

      integer  ns,i,m 
      real  dif,dift,ho,hp,w,den 

      NS=1
      DIF=ABS(XA(1))

      DO I=1,N
         DIFT=ABS(XA(I))
         IF (DIFT.LT.DIF) THEN
            NS=I
            DIF=DIFT
         END IF
         C(I)=YA(I)
         D(I)=YA(I)
      ENDDO 

      Y=YA(NS)
      NS=NS-1

c     Initial value of Y 

      DO M=1,N-1
         DO I=1,N-M
            HO=XA(I)
            HP=XA(I+M)          
c no x ne 0 offset   
            W=C(I+1)-D(I)
            DEN=HO-HP
            IF (DEN.EQ.0.0) then
               write(*,*) ' ERROR in Polint '
               stop
            END IF
            DEN=W/DEN
            D(I)=HP*DEN
            C(I)=HO*DEN
         ENDDO
         IF (2*NS.LT.N-M)THEN
            DY=C(NS+1)
         ELSE
            DY=D(NS)
            NS=NS-1
         END IF
         Y=Y+DY
      ENDDO
      DY = ABS(DY)

c New value, and error estimate 

      return
      end subroutine Polint

      Function Funcs (X, Switch)

c     The integrand in the integral to be computed.
c     Switch controls which integrand is used 
c---------------------------------------------------

      include 'global.h'
      real  X,dd,fdmu,Funcs  
      integer  Switch 
      
      IF (Switch.eq.1) THEN 
c        Linear Limb-darkening 
c        funcsv1 = mustar_r 
c        funcsv2 = sigma_r 
c        funcsv3 = alpha_r 
         dd   = 0.5 + 3./4.*sqrt( (X**2-funcsv1**2)/(1.-funcsv1**2) )
         fdmu = dd*X*(1.+funcsv2*mu**2)**funcsv3
         Funcs = fdmu
         return
      ELSE IF(Switch.eq.2) THEN
c        Test case for no limb-darkening, should return analytic expression
         dd   = 1.0
         fdmu = dd*X*(1.+funcsv2*mu**2)**funcsv3
         Funcs = fdmu
         Return 
      ELSE 
         stop'Only Switch < 3 allowed!'
      ENDIF
     
      end Function Funcs
