      subroutine init(regrid,dxfac)
C
C Initialize Time-Dependent Stellar Wind problem
C This version takes a VH-1 input file;
C    the grid may be used as is (regrid = 'no' in INDAT),
C    or as set up by GRIDX      (regrid = [anything else] in INDAT).
C 11jun92 cooper, owocki
C=======================================================================
C
C GLOBALS
C
      include 'global.h'
      include 'zone.h'
C
C=======================================================================
C
C constants that define problem
C
      character*2 regrid
      parameter (xlsun=3.8268e33,xmsun=1.9891e33,rsun=6.9599e10)
      parameter (pi=3.141592654,tpi=2.*pi,fpi=4.*pi)
      parameter (clight=2.997925e10,sigel=0.66524e-24)
      parameter (gravcon=6.6704e-8)
      parameter (xhabun=0.73,yheabun=1.-xhabun)
      parameter (xmpro  = 1.67266e-24)
      parameter (xmolwt= 2./(1.+3.*xhabun+0.5*yheabun))
      parameter (elkap=0.2*(1.+xhabun))
      parameter (gamsun=elkap*xlsun/(clight*fpi*gravcon*xmsun))
      parameter (wpvamp=0.,wpv=1.e7 ,wpvsig=1.e7 )
      parameter (wpzamp=0.,wpz=1.e12,wpzsig=1.e12)
      real gamdum
      real xmdcak_fd, xmdcak_fd_vth, delm, vesc, Qbar_test 
c     data time   /0.0/
C Set up geometry and boundary conditions of grid
C
C Boundary condition flags : nleft, nright
C   = 0  :  reflecting boudary condition
C   = 1  :  inflow/outflow boundary condition
C   = 2  :  fixed inflow boundary condition
C   = 5  :  ``special'' boundary (cook up your own)
C Geometry flag : ngeom
C   = 0  :  planar
C   = 1  :  cylindrical radial
C   = 2  :  spherical   radial
C   = 3  :  cylindrical angle
C   = 4  :  spherical   polar angle
C
      ndim   = 1
      ngeomx = 2
      ngeomy = 0
      ngeomz = 0
      nleftx = 5
      nrightx= 5
      nlefty = 0
      nrighty= 0
      nleftz = 0
      nrightz= 0
c
      gamm   = gam - 1.
      boltzman = 1.380626e-16
      avgmass  = xmpro*xmolwt
c       xlpert= 0.4*sin(tpi*(time/xlper+xlphase))
      xlstar = xlsbs*xlsun
      xmstar   = xmsbs*xmsun
c      GM = gravcon*xmstar*(1.-gamsun/xmsbs)
c  TEST JS -- BUG -------
      gamdum = (1.-xlsbs*gamsun/xmsbs)
      IF (gamdum.LT.0.0) Stop'Gamma > 1 !' 
      GM = gravcon*xmstar*gamdum
c -----------------------------------------
      rstar = rsbs*rsun
      asndw  = sqrt(boltzman*tempw/avgmass)
      vtherm = 10.e5
      if (vthba.gt.0.) vtherm = vthba*asndw
c      delv_cut = asndw
      delv_cut = vtherm 
      oma    = 1. - abbott
      opa    = 1. + abbott
      gap1   = exp(gammln(abbott + 1.))
c OBS --- NEW, NEW, NEW !! 
c ----------------------------------------
c TRANSLATION FROM NEQ Qbar INPUT, JS1203 
      xkovbc = ( (Qbar*elkap)**oma/exp(gammln(abbott)) )**(1./oma)
c-----------------------------------------
      cak = xlstar*gap1*(xkovbc**oma)
      cak = cak/(fpi*abbott*oma*clight**opa)
      xkapo = xkovbc*clight/vtherm
      xkapm = xkapo*xkmbko
      gamo  = xkovbc*xlstar/(fpi*GM*clight)
      xmdpho= xlstar/clight**2
c      xmdcak= xmdpho*gap1**(1./abbott)*(abbott*gamo)**(oma/abbott)/oma
c ALSO A BUG  --------------------
c JS 
      xmdcak= xmdpho*gap1**(1./abbott)*(gamo/abbott)**(oma/abbott)/oma
c ADDED FINITE DISC+FINITE THERMAL SPEED CORRECTION 
c +TRANSLATION TO GAYLEY 95 Qbar FORMALISM, JS 0312 
      xmdcak_fd = xmdcak/opa**(1./abbott)
      vesc = 2.*gravcon*xmstar*gamdum/(rsbs*rsun)
      vesc = sqrt(vesc)
      delm = 4.*sqrt(1.-abbott)/abbott * (asndw/vesc)
      xmdcak_fd_vth = xmdcak_fd*(delm+1.)
      Qbar_test = ( xkovbc**oma*exp(gammln(abbott)) )**(1./oma)/elkap
      write(*,*),'Test so Qbar translation went smoothly, '
      write(*,*),'This should be equal to input Qbar: ',Qbar_test
c---------------------------------------------------
      uinflo = 0.0
      vinflo = 0.0
      winflo = 0.0
      ein0   = boltzman * tempw / gamm / avgmass
      einflo = ein0 + 0.5*(uinflo*uinflo + vinflo*vinflo
     &                                   + winflo*winflo)
C
C=======================================================================
C
C initial state  -  read from file vh1.init
C
      secbyr = 3.1557e7
      write (8,*) 
      write (8,*) 'Wind initialized from vh1.init'
      write (8,*) 
      write (8,*) 'Stellar parameters: lum  = ',xlsbs
      write (8,*) '                    mass = ',xmsbs
      write (8,*) '                  radius = ',rsbs
      write (8,*) '                    temp = ',tempw
      write (8,*) 'Adiabatic index,   gamma = ',gam
      write (8,*) 'Line parameters:  abbott = ',abbott
      write (8,*) '                  xkovbc = ',xkovbc
      write (8,*) '                  xkmbko = ',xkmbko
      write (8,*) '                   vthba = ',vthba
      write (8,*) '                  vtherm = ',vtherm
      write (8,*) '                   asndw = ',asndw
      write (8,*) '               Eddington = ',1.-gamdum
      write (8,*) 'Effective escape speed           vesc   = ',vesc
      write (8,*) 'Gayley -95 Qbar             :    Qbar   = ',Qbar
      write (8,*) 'CAK point-source mass flux  :    xmdcak = ',xmdcak*secbyr/xmsun
      write (8,*) 'CAK finite-disc mass flux   :    xmdcak = ',xmdcak_fd*secbyr/xmsun
      write (8,*) 'CAK fd+vth (OuD04)mass flux :    xmdcak = ',xmdcak_fd_vth*secbyr/xmsun      

      write (8,*)
C
C########################################################################
C
      call vh1init(regrid,dxfac)
c NOW GRID IS FIXED -- I can invoke radial dependence of Qbar, i.e.
c xkovbc = k0*vth/c, which has the same dependence 
c I'll use a parameter koind as power-index (default = 0,-1) for now 
c
c     at ZONE INTERFACES -- change to centers later?? 
c      cak = xlstar*gap1*(xkovbc**oma)
c      cak = cak/(fpi*abbott*oma*clight**opa)
c             xmustar   = sqrt(1.-(rstar/zxa(i))**2)
c
c      do i=1,nrow 
c         cak(i,1,1) = xkovbc*(zxa/rstar)**koind
c         cak(i,1,1) = xlstar*gap1*cak(i,1,1)**oma
c         cak(i,1,1) = cak(i,1,1)/(fpi*abbott*oma*clight**opa)
c      enddo
c
c esfrf will be a little problem --> cak -> bvth -> tmp -> 
c make it only in Sobo for now perhaps... if (ifrc.eq.0.and. ...) ? 
c
      if (dinflo.le.0.) dinflo = zrho(1,1,1)
c ADDITION JS -- MULTIPLY BY factor of three..
      if (dinflo.gt.10.) dinflo = zrho(1,1,1)*3.0
      pinflo = boltzman * tempw / avgmass * dinflo
      if ((time.lt.small).or.(xmlag(1).lt.0.)) then
        xmlag(imax)=0.
        do i=imax,2,-1
           atmp = 1.
           if (ngeomx.eq.1) atmp=(zxa(i)+0.5*zdx(i))
           if (ngeomx.eq.2) atmp=(zxa(i)+0.5*zdx(i))**2
           xmlag(i-1) = xmlag(i)+zrho(i,1,1)*atmp*zdx(i)
      zrho(i,1,1) = zrho(i,1,1)*(1.
     $ + wpvamp*exp(-((zux(i,1,1)-wpv)/wpvsig)**2)
     $ + wpzamp*exp(-((zuz(i,1,1)-wpz)/wpzsig)**2)
     $  )
        enddo
      endif

      write (8,*) ',dinflo = ',dinflo 
C
      return
      end
C
C=======================================================================
C//////////////////////////  SUBROUTINE VH1INIT \\\\\\\\\\\\\\\\\\\\\\\
C
      subroutine vh1init (regrid,dxfac)
C
      include 'global.h'
      include 'zone.h'
C
      character*2 regrid
      integer i,j,k,n,nrow
      parameter (nrmax=2603)
      real xin(nrmax),vin(nrmax),din(nrmax)
      real xmlin(nrmax),pin(nrmax),soin(nrmax)
      external gammln,vh1rd
c
      call vh1rd (nrow,xin,vin,din,xmlin,pin,soin)
c
      d2p = boltzman * tempw / avgmass
      j = 1
      k = 1
c
      if (regrid .eq. 'no') then
c	 igridfac = nrow / imax
	 igridfac = 1
c	 if (igridfac*imax .ne. nrow) then
c	    write (*,*) '# points is not an integer fraction of input'
c	    stop
c	 endif
	 do i = 1, nrow
	    i1 = igridfac*(i-1) + 1
	    zxa (i)     = xin(i1)
	    zux (i,j,k) = vin(i1)
	    zuy (i,j,k) = 0.
	    zuz (i,j,k) = 0.
	    zrho(i,j,k) = din(i1)
	    xmlag(i)    = xmlin(i1)
	    zp  (i,j,k) = pin(i1)
	    zso (i,j,k) = soin(i1)
	 enddo
	 do i = 1, nrow-1
	    zdx (i)     = zxa(i+1) - zxa(i)
	 enddo
c	 zdx(nrow) = zdx(nrow-1)	! Better: store zdx(imax) in fin file
 	 zdx(nrow) = zdx(nrow-1)*dxfac	! Better: store zdx(imax) in fin file
	 do i = nrow+1, imax
c	    zdx(i) = zdx(nrow)
	    zdx(i) = zdx(i-1)*dxfac
	    zxa(i) = zxa(i-1) + zdx(i)
	    zux (i,j,k) = zux(nrow,j,k)
	    zuy (i,j,k) = 0.
	    zuz (i,j,k) = 0.
	    zrho(i,j,k) = zrho(nrow,j,k)*(zxa(nrow)/zxa(i))**2
            atmp = 1.
            if (ngeomx.eq.1) atmp=(zxa(i)+0.5*zdx(i))
            if (ngeomx.eq.2) atmp=(zxa(i)+0.5*zdx(i))**2
            xmlag(i) = xmlag(i-1)-zrho(i,1,1)*atmp*zdx(i)
c	    zp  (i,j,k) = zp(nrow,j,k)*(zxa(nrow)/zxa(i))**2
	    zp  (i,j,k) = zrho(i,j,k)*tempw*boltzman/avgmass
c	    zso (i,j,k) = zso(nrow,j,k)
            xmustar   = sqrt(1.-(rstar/zxa(i))**2)
            zso (i,j,k) = 0.5/(1.+xmustar)
	    xmax = zxa(i)
	 enddo
c
      else
c
         do i = 1, imax
c	    xmlag(i) = -1.		! reset xmlag rather than interpolate
	    if (zxa(i) .lt. xin(1)) then
	       delxrat = (xin(1) - zxa(i)) / (xin(2) - xin(1))
	       zux (i,j,k) = vin(1) + delxrat*(vin(1)-vin(2))
	       zuy (i,j,k) = 0.
	       zuz (i,j,k) = 0.
	       zrho(i,j,k) = din(1) + delxrat*(din(1)-din(2))
	       zp  (i,j,k) = pin(1) + delxrat*(pin(1)-pin(2))
	       zso (i,j,k) =soin(1) + delxrat*(soin(1)-soin(2))
	       xmlag(i)  = xmlin(1) + delxrat*(xmlin(1)-xmlin(2))
	    else if (zxa(i) .gt. xin(nrow)) then
	       zux (i,j,k) = vin(nrow)
	       zuy (i,j,k) = 0.
	       zuz (i,j,k) = 0.
	       zrho(i,j,k) = din(nrow) * (xin(nrow)/zxa(i))**2
	       zp  (i,j,k) = d2p * zrho(i,j,k)
	       zso (i,j,k) = soin(nrow)
               atmp = 1.
               if (ngeomx.eq.1) atmp=(zxa(i)+0.5*zdx(i))
               if (ngeomx.eq.2) atmp=(zxa(i)+0.5*zdx(i))**2
               xmlag(i) = xmlag(i-1)-zrho(i,1,1)*atmp*zdx(i)
            else
	       do i2 = 1, nrow-1
		  if (zxa(i) .le. xin(i2+1)) then
		     delxrat = (zxa(i)-xin(i2)) / (xin(i2+1)-xin(i2))
		     zux (i,j,k) = vin(i2) + delxrat*(vin(i2+1)-vin(i2))
		     zuy (i,j,k) = 0.
		     zuz (i,j,k) = 0.
		     zrho(i,j,k) = din(i2) + delxrat*(din(i2+1)-din(i2))
		     zp  (i,j,k) = pin(i2) + delxrat*(pin(i2+1)-pin(i2))
		     zso (i,j,k) =soin(i2) 
     &                              + delxrat*(soin(i2+1)-soin(i2))
		     xmlag(i)  = xmlin(i2) 
     &                              + delxrat*(xmlin(i2+1)-xmlin(i2))
		     goto 10
		  endif
	       enddo
	          write (*,*) 'Interpolation in VH1INIT failed: i=',i
10             continue
	    endif
	 enddo
      endif
c
c      do i2=1,nrow 
c         write(*,*) zxa(i2)/rstar, zso(i2,j,k) 
c      enddo
c      stop'Testing s!'
      return
      end
C
C=======================================================================
C//////////////////////////  SUBROUTINE VH1RD \\\\\\\\\\\\\\\\\\\\\\\
C
      subroutine vh1rd(nrow,xin,vin,din,xmlin,pin,soin)
C
C  Reads in wind initial condition from VH1 .fin file
C
      include 'global.h'
C
      integer i,ir,ncol,nrow,nrmax
      real r,v,d,xm,p
      real rsclin,vsclin,dsclin,xmsclin,timsclin,psclin,ssclin,xmustar
      parameter (nrmax=2603)
      real xin(nrmax),vin(nrmax),din(nrmax)
      real xmlin(nrmax),pin(nrmax),soin(nrmax)
C
      open (unit=81, file='vh1.init')
      read(81,*) ncol,nrow,ncycle,time,rsclin,vsclin,dsclin,
     $                xmsclin,timsclin,psclin,ssclin
c
c     open (unit=83, file='spord.tst')
c
      nrow = min(nrow,nrmax)
      do i=1,nrow
	 so = 0.
	 if (ncol.eq.5) read (81,*) ir,r,v,d,xm,p
	 if (ncol.ge.6) read (81,*) ir,r,v,d,xm,p,so
         if (rsclin.gt.0.) then
            xin(i) =  rsclin * (1. + r)
         else
            xin(i) = -rsclin*r
         endif
         vin(i) = v*vsclin
         din(i) = d*dsclin
	 pin(i) = p*psclin
	 xmlin(i)= xm*xmsclin
	 soin(i) = so
c ADDED OPTION, JS 0312 
c Use default optically thin source function option here
c         if (ssclin.ge.1.0) then 
c            xmustar     = sqrt(1.-(rstar/xin(i))**2)
c            soin(i) = 0.5/(1.+xmustar)         
c         endif
c-----------------------------------------------
c     write (83,8383) i, xin(i),vin(i),din(i),xmlin(i),pin(i)
c8383  format (i6, 1p5e13.5)
c
      enddo
      close (81)
c     close (83)
      return
      end
c
c---------------------------------------------------------------------------
c
      real function gammln(xx)
c
c     Returns the log of complete gamma function, ln (gamma(x+1)) == ln (x!).
c     Reference: Numerical Recipes, Press et al. ch. 6.
c
      implicit NONE
      integer j
      real xx,x,one,half,fpf,cof(6),stp,tmp,ser
      data cof,stp /76.18009173e0,-86.50532033e0,24.01409822e0,
     $             -1.231739516e0, 1.20858003e-3,  -5.36382e-6,
     $            2.50662827465e0/
      parameter (one=1.0,half=0.5,fpf=5.5)
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      x = xx-one
      tmp = x+fpf
      tmp = (x+half)*log(tmp)-tmp
      ser = one
      do j=1,6
        x = x+one
        ser = ser+cof(j)/x
      enddo
      gammln = tmp + log(stp*ser)
      return
      end
