      program vhone
C
C--------------------------------------------------------------------------
C
C
C          VV              VV   HH       HH         111
C           VV            VV    HH       HH        1111
C            VV          VV     HH       HH          11
C             VV        VV      HH       HH          11
C              VV      VV       HHHHHHHHHHH   ==     11
C               VV    VV        HH       HH          11
C                VV  VV         HH       HH          11
C                 VVVV          HH       HH          11
C                  VV           HH       HH        111111
C
C 
C                        VIRGINIA HYDRODYNAMICS #1
C
C--------------------------------------------------------------------------
C  The Virginia Numerical Bull Session ideal hydrodynamics PPMLR
C  
C  Version 1.0 October 1991 
C  This version was removed from the ``loop'' in the continued development
C  of VH-1 on Sep 1 1990, but has followed a similar evolution until this
C  release date.
c
c  Revision history:
c97: 24-Nov: Add ifrc=7, for eisf w/ tauc; add nzts & irssfmax in namelist
c94: 30-Sep: Add ifrc=4,5,6 options.
c92: 15-Jun: New init.f to read & write TDSW-style files
c          : Added variable xmlag (evolve.f, zone.h)
c          : Added many variables to namelist hinput (vhone.f)
c          : Added many quantities to common (global.h)
c          : Replaced sobolev.f with cakfrc.f (if vthbc =0) & ssffrc.f (if >0)
c          : Added source fn variable xso (init.f, tdpltprn.f, zone.h)
c          : Set xso = -epsabs -1 when epsabs <0 (init.f, ssffrc.f)
c          : Made dinflo an input; use input value if >1.e10, else use first
c          :      rho value from init file  (init.f, vhone.f, indat)
c    16-Dec: Added const. time step dtmax (dtcon.f, global.h)
c    22-Dec: Moved inputs amp, phase, period to namelist hinput (sweepbc.f,
c             vhone.f, global.h, init.f)
c          : Moved input issfrbc to hinput (ssffrc.f, vhone.f, global.h, init.f)
c93:  1-Jul: Changed vhone.f to write & plot at time tprnxt, so output times
c                 stay close to intended times.
C 
C Output channels:
C                 4 = aaaaadaa,   binary dump files for restarting
C                 8 = aaaaahst,   history file
C                     aaaaamvy,   HDF raster movie file
C                     aaaaa.1000, HDF SDS output
C GLOBALS
C
      include 'global.h'
      include 'zone.h'
      include 'sweepsize.h'
C
C LOCALS
C
c      external sweepx, sweepy, dtcon, grid, init, dump, images
      external sweepx, sweepy, dtcon, gridx, gridy, gridz, init, dump
C
      character dmpfile*8, hstfile*7, mvyfile*8,prtfile*7,
     &          prefx*5, rstrt*2, regrid*2, sf1*1, sf2*1, char*1
      real endtime,tprnxt,tprin,xmin,xmax,ymin,ymax,zmin,zmax
      real nfcl,norm,fcl,t1,tst1
      integer ncycp, ncycd, ncycm, ncycend, nprin, ndump, 
     &        nmovie, tmpcyc
      integer stdnew, stdout, ncl,ncyc_fcl
      parameter (rsun=6.9599e10)
C
C Namelist for input deck
C
      namelist / hinput / rstrt, regrid, prefx
     &           , xmin, xmax, ymin, ymax,zmin, zmax
     &           , imax, jmax, kmax, dxfac, dyfac, dzfac
     &           , xmsbs,xlsbs,rsbs,tempw,dinflo
     &           , gam, xlamfac, courant, dtmax
     &           , abbott,xkovbc,xkmbko,vthba,epsabs,Qbar
     &           , ncycend, ndump, nprin, nmovie, tprin, endtime
     &           , amp, phase, period, xlamp, xlphase, xlper 
     &           , issfrbc,ifrc,nzts,irssfmax,neg_frad

C
C Set some floor values for pressure and density 
C
      data  small   / 1.e-35 /
      data  smallp  / 1.e-35 /
      data  smallr  / 1.e-35 /
C
C Define unit numbers for files vh1.fin and vh1.out. and vh1.stat JS1204 
C
      data stdnew / 11 /
      data stdout / 12 /
c      data stdstat / 13 /
C Adding another output file, JS 1211
      open (unit=18,file='vh1.time')
C
C-----------------------------------------------------------------------
C
C Begin by reading input deck, and defining file names
C
      ncyc_fcl = 30000
c      t_fcl = 30.
      t_fcl = 100000.
      nfcl = 0.0
      rhoav = 0.0
      rhoav2 = 0.0
      vav = 0.0
      vav2 = 0.0
      logrhoav = 0.0
      logrhoav2 = 0.0
      vlogrhoav = 0.0
c -- NEW JS - WHERE TO START COUNTING CLUMPING FACTORS? 
      gam   = 1.66666667
      courant= 0.5
      xlamfac= 1.
      dxfac = 1.
      dyfac = 1.
      dzfac = 1.
      xmsbs = 50.
      xlsbs = 7.e5
      rsbs  = 19.
      tempw = 50000.
      dinflo= 10
      vthba = 0.
      abbott= 0.7
c      xkovbc=2270.
      Qbar = 2000.
      xkmbko=1.e-3
      epsabs=0.
      nzts  =10
      irssfmax=-1
      amp   = 0.
      phase = 0.
      period= 1.
      xlamp = amp
      xlphase=phase
      xlper  = period
      neg_frad = 0
      open (unit=15,file='indat',status='old',form='formatted')
      read (15,hinput,err=999,end=999)
      close(15)
      if (xmin.lt.1.e10) xmin=xmin*rsbs*rsun
      if (xmax.lt.1.e10) xmax=xmax*rsbs*rsun
c      prtfile = prefx(1:5) // 'srf'
c      hstfile = prefx(1:5) // 'hst'
      prtfile = 'vh1.out'
      hstfile = 'vh1.hst'
      dmpfile = prefx(1:5) // 'daa'
      mvyfile = prefx(1:5) // 'mvy'
C
C Check that arrays are large enough for desired number of physical zones
C
      if (imax .gt. II) then
         write(*,*) 'II less than imax = ', imax, II
         stop
      endif
      if (jmax .gt. JJ) then
         write(*,*) 'JJ less than jmax = ', jmax, JJ
         stop
      endif
      if (kmax .gt. KK) then
         write(*,*) 'KK less than kmax = ', kmax, KK
         stop
      endif
      if (max(imax,jmax,kmax)+4 .gt. maxsweep) then
         write(*,*) 'maxsweep too small'
         stop
      endif
C
C Open print file for later writes from PRIN
C Open history file and write out a description of the run
C
      open (unit=stdout,file=prtfile,form='formatted')
      open (unit=8,file=hstfile,form='formatted')
      open (unit=9,file='vh1.ts',form='formatted')
      write (8,*) 'History File for VH-1 run'
      write (8,911) prefx
      write (8,913) mvyfile
 911  format(' Filename prefix: ',a5)
 913  format(' HDF movie filename: ',a10)
      write (8,*) 'Courant number  = ',courant
      write (8,*) 'Grid dimensions: imax= ',imax,' jmax= '
     &                      ,jmax,' kmax= ',kmax
      write (8,*) 'Outer x location = ',xmax
      write (8,*) 'Inner x location = ',xmin
      write (8,*) 'Outer y location = ',ymax
      write (8,*) 'Inner y location = ',ymin
      write (8,*) 'Outer z location = ',zmax
      write (8,*) 'Inner z location = ',zmin
      write (8,*) ' '
      write (8,hinput)
      write (8,*) ' '
C
      if ((rstrt.eq.'NO').or.(rstrt.eq.'no')) then
C
C Initialize variables for new problem; 
C
         ncycle = 0
         ncycp  = 0
         ncycd  = 0
         ncycm  = 0
         nfile  = 1000
C
         call gridx(imax,xmin,xmax,zxa,zdx,dxfac)
         call gridy(jmax,ymin,ymax,zya,zdy,dyfac)
         call gridz(kmax,zmin,zmax,zza,zdz,dzfac)
         call init (regrid,dxfac)
         dt = sqrt(zp(1,1,1)*gam/zrho(1,1,1))
         call dtcon
         call siminit
         tprnxt = time
c         write(6,*)'DHC (vhone): call to tdpltprn at top'
         call tdpltprn(stdnew,stdout)
         tprnxt = tprnxt + tprin
      else
C
C Restart from old dumpfile...
C
         dmpfile = dmpfile(1:6) // rstrt(1:2)
         write(8,*) 'Restarting from ',dmpfile
         open(unit=4,file=dmpfile,status='old',form='unformatted')
         tmpcyc=ncycend
         read(4,err=998) ADUMP, BDUMP
         close(4)
         ncycend=tmpcyc
C
C Increment dump filename
C
         if(dmpfile(8:8) .eq. 'z' .or. dmpfile(8:8) .eq. 'Z') then
            sf1  = dmpfile(7:7)
            isf1 = ichar(sf1) + 1
            sf2  = dmpfile(8:8)
            isf2 = ichar(sf2) - 25
            dmpfile(7:7) = char(isf1)
            dmpfile(8:8) = char(isf2)
         else 
            sf2  = dmpfile(8:8)
            isf2 = ichar(sf2) + 1
            dmpfile(8:8) = char(isf2)
         endif
C
       call siminit
       endif
C
      write (*,*) 'Starting on cycle number:  ', ncycle
C
C############################################################################
C
C                         MAIN COMPUTATIONAL LOOP
 1    continue
C
c         write(6,*)'DHC (vhone): entered main loop'

      if ( time + dt .gt. endtime ) then
C                                        go exactly to the end
         dt = endtime - time
         ncycend = ncycle
         ncycp   = nprin
         ncycd   = ndump
c         write(*,*)'DHC (vhone): into endtime if block  at top, ncy* set:'
c         write(6,*)'DHC (vhone): ncycp, ncycend, ncycd,time', ncycp, ncycend, ncycd,time
      endif

c         write(6,*)'DHC (vhone): passed endtime check at top, ncy* set:'

c         write(6,*)'DHC (vhone): ncycp, ncycend, ncycd,time', ncycp, ncycend, ncycd,time

c         write(6,*)'DHC (vhone):   '
C
C
C
C Evolution step: PPM - 3rd order piecewize parabolic. 
C         
      if (ndim .eq. 1) then
c         write(*,*),'test!!',zux(1,1,1)
c         print*,time,'test1'
         call sweepx
c         print*,time,'test2'
c         write(*,*),'test22!!',zux(1,1,1)
c
c-------------------------------------------
         call radiate
c TURNED OFF RAD COOLING JS & DC 2013-04 
c-------------------------------------------
c
c         write(6,*)'DHC (vhone): about to increment time and call dtcon'
c         write(6,*)'DHC (vhone): time, dt:', time, dt
         time  = time  + dt
         call dtcon
c         write(6,*)'DHC (vhone): just after call to dtcon'
c         write(6,*)'DHC (vhone): time, dt:', time, dt
c         write(6,*)' '
C
C
      else if (ndim .eq. 2) then
C
C In two dimensions sweep XYYX
C
         if ((ncycle - 2*(ncycle/2)) .eq. 0) then
            call sweepx
            call sweepy
            call radiate
            time  = time  + dt
         else
            call sweepy
            call sweepx
            call radiate
            time  = time  + dt
            call dtcon
         endif
      else if (ndim .eq. 3) then
C
C In three dimensions sweep XYZZYX
C
         if ((ncycle - 2*(ncycle/2)) .eq. 0) then
            call sweepx
            call sweepy
            call sweepz
            call radiate
            time  = time  + dt
         else
            call sweepz
            call sweepy
            call sweepx
            call radiate
            time  = time  + dt
            call dtcon
         endif
      endif
C                                        Loop Bookkeeping
      ncycle = ncycle + 1
      ncycp  = ncycp  + 1
      ncycd  = ncycd  + 1
      ncycm  = ncycm  + 1
C                                        call routine to print movie images
      if ( ncycm.ge.nmovie ) then
c         call images(mvyfile,1)
         ncycm = 0
      endif
C                                  call routine to print out numbers
c ADDED JS-03111
c--------------------------------------------------
c      if (ncycle.gt.ncyc_fcl) then 
c      if (time.gt.t_fcl) then 
c         do ncl=1,imax 
c            rhoav(ncl)  = rhoav(ncl)  + zrho(ncl,jj,kk)*dt  
c            rhoav2(ncl) = rhoav2(ncl) + zrho(ncl,jj,kk)*zrho(ncl,jj,kk)*dt  
c            vav(ncl)    = vav(ncl)    + zux(ncl,jj,kk)*dt
c            vav2(ncl)   = vav2(ncl)   + zux(ncl,jj,kk)*zux(ncl,jj,kk)*dt
c         enddo 
c         nfcl = nfcl + 1.0
c      endif
      if (time.gt.1.8e5.and.time.le.2.0e5) then 
         do ncl=1,imax 
            if (zxa(ncl)/rstar.gt.2.0) then 
               write(18,188) time, zxa(ncl)/rstar, zrho(ncl,1,1), zux(ncl,1,1), zp(ncl,1,1)
               exit
            endif
         enddo
      endif 
 188  FORMAT(5e20.7)

      if (time.gt.t_fcl) then 
         if (nfcl.lt.0.1) then
            do ncl=1,imax 
               yold1(ncl)  = zrho(ncl,jj,kk)
               yold2(ncl)  = zrho(ncl,jj,kk)*zrho(ncl,jj,kk)  
               yold3(ncl)  = zux(ncl,jj,kk)
               yold4(ncl)  = zux(ncl,jj,kk)*zux(ncl,jj,kk)
c NEW coorelation parameters, cf Runacres & Owocki 2002
               yold5(ncl)  = alog10(zrho(ncl,jj,kk))
               yold6(ncl)  = alog10(zrho(ncl,jj,kk))**2
               yold7(ncl)  = zux(ncl,jj,kk) * alog10(zrho(ncl,jj,kk))
            enddo 
            t_fcl = time 
         else
            do ncl=1,imax 
               rhoav(ncl)  = rhoav(ncl) + 0.5*( zrho(ncl,jj,kk)+yold1(ncl) )*dt  
               yold1(ncl)  = zrho(ncl,jj,kk)

               rhoav2(ncl) = rhoav2(ncl) + 0.5*(zrho(ncl,jj,kk)*zrho(ncl,jj,kk) + yold2(ncl) )*dt  
               yold2(ncl)  = zrho(ncl,jj,kk)*zrho(ncl,jj,kk)

               vav(ncl)    = vav(ncl) + 0.5*(zux(ncl,jj,kk) + yold3(ncl) )*dt
               yold3(ncl)  = zux(ncl,jj,kk)

               vav2(ncl)   = vav2(ncl) + 0.5*(zux(ncl,jj,kk)*zux(ncl,jj,kk) + yold4(ncl) )*dt
               yold4(ncl)  = zux(ncl,jj,kk)*zux(ncl,jj,kk)

               logrhoav(ncl)  = logrhoav(ncl) + 0.5*( alog10(zrho(ncl,jj,kk))+yold5(ncl) )*dt  
               yold5(ncl)  = alog10(zrho(ncl,jj,kk))

               logrhoav2(ncl)  = logrhoav2(ncl) + 0.5*( alog10(zrho(ncl,jj,kk))**2+yold6(ncl) )*dt  
               yold6(ncl)  = alog10(zrho(ncl,jj,kk))**2

               vlogrhoav(ncl)  = vlogrhoav(ncl) + 0.5*( zux(ncl,jj,kk)*alog10(zrho(ncl,jj,kk))+yold7(ncl) )*dt  
               yold7(ncl)  = zux(ncl,jj,kk)*alog10(zrho(ncl,jj,kk)) 

            enddo 
         endif
         nfcl = nfcl + 1.0
      endif
c-----------------------------------------------------
      if ( ncycp.ge.nprin .or. time.ge.tprnxt ) then
c         write(6,*)'DHC (vhone): call to tdpltprn at bottom'
c         write(6,*)'DHC (vhone): ncycp, nprin, time, tprnxt', ncycp, nprin, time, tprnxt  
         call tdpltprn(stdnew,stdout)
         tprnxt = tprnxt + tprin
         ncycp = 0
      endif
C                                        dump out a complete dataset
      if ( ncycd .ge. ndump ) then
         call dump(dmpfile)
         ncycd = 0.
      endif
c
c     dhc 17jan08 - commented out following few lines
c 
C					 print out time series at selected ir.
c      if(nzts.gt.0) then
c        idel=imax/nzts
c        write(9,991) time
c     $            ,zux(1,1,1),zrho(1,1,1)
c     $            ,((zux(i*idel,1,1),zrho(i*idel,1,1)),i=1,nzts)
c991   format(1pe15.7,20(/1x,1p8e14.6))
c      endif
C
c
      if ( ncycle.lt.ncycend ) goto 1
C
C                           END OF MAIN LOOP
C#########################################################################
C
      write(*,*) 'Time at end of run = ', time
c---- new!! print some clumping factors, 1st attempt JS-0311
c
cc      open(33,file='vh1.rhoav')
cc      norm = float(ncycend-ncyc_fcl)
cc      do ncl=1,imax 
ccc         if (nfcl.gt.0.0) then
cc         if (time.gt.t_fcl) then 
ccc         write(33,*) rhoav(jjj)/norm,rhoav2(jjj)/norm
cc            rhoav2(ncl) = rhoav2(ncl)/(time-t_fcl)
cc            rhoav(ncl)  = rhoav(ncl)/(time-t_fcl)
cc            fcl = rhoav2(ncl)/rhoav(ncl)**2
ccc            write(33,32) zxa(ncl)/rstar-1.,rhoav(ncl)/nfcl,rhoav2(ncl)/nfcl,fcl
cc            write(33,32) zxa(ncl)/rstar-1.,rhoav(ncl),rhoav2(ncl),fcl
cc         else
cc            write(33,32) zxa(ncl)/rstar-1.,0.0
cc         endif
cc      enddo
cc      close(33)
cc 32    FORMAT(4e15.5)
c
c
      close( 8 )
      close( stdout )
      close( 18 ) 
      call simfin
C
c
      call cpu_time(t1) 
      write(*,*),'CPU-seconds used: ',t1
      write(*,*),'NORMAL END' 
      stop
998   write(*,*) 'Error reading restart dump'
      stop
999   write(*,*) 'Error reading namelist file'
      stop
      end
