C zone.h -- common block for 2D (3D) arrays
C
C 09jun90 gbl
C 21jun90 jmb delete zeps
C jul 90 jfh 2D version
C jan 92 jmb 3D version
C
C parameters:
C  ii - number of zones in x direction
C  jj - number of zones in y direction
C  kk - number of zones in z direction
C  ig - number of zones in image array (ig X ig)
C
C common block variables:
C  zrho  - density: zone average
C  zp    - pressure: zone average
C  zux   - zone average velocity x direction
C  zuy   - zone average velocity y direction
C  zuz   - zone average velocity z direction
C  zxa   - x grid zone face locations
C  zya   - y grid zone face locations
C  zza   - z grid zone face locations
C  zdx   - xa(i+1) - xa(i)
C  zdy   - ya(i+1) - ya(i)
C  zdz   - za(i+1) - za(i)
C  imax  - number of physical zones in the x direction
C  jmax  - number of physical zones in the y direction
C  kmax  - number of physical zones in the z direction
C  ntotal- total length of stuff to be stored in restart file
C  adump - vector to hold as much memory as needed to for restart
c  zgr   - zone centered total radiative force
c  zgs   - zone centered scattered force
c xmlag  - lagrangian mass (1-D) (SPO 6/15/92)
c nx     - # obs. frame frequencies (SPO 6/17/92)
c xvec   - frequency grid for SSF force (SPO 6/17/92)
c esumr  - right boundary opt. depth (SSF) (SPO 6/17/92)
C
      integer ii,jj,kk,ig,imax,jmax,kmax,ntotal
C
C Note that ii, jj, and kk must be at least as big as the input imax,jmax,kmax
C
      parameter ( ii = 8200 )
      parameter ( jj =    1 )
      parameter ( kk =    1 )
      parameter ( ig =    1 )
      parameter (nxmax=8200)
      parameter ( ntotal = 8*ii*jj*kk+3*ii+2*jj+2*kk+2+2*ig*ig+nxmax+1)
C
      real zrho(ii,jj,kk), zp(ii,jj,kk), zux(ii,jj,kk), zuy(ii,jj,kk)
     &     ,zuz(ii,jj,kk), zgr(ii,jj,kk),zgs(ii,jj,kk),zxa(ii), zdx(ii)
     &     ,zya(jj), zdy(jj), zza(kk), zdz(kk), xmlag(ii),zso(ii,jj,kk)
     &     ,ztauc(ii,jj,kk),zgssf(ii,jj,kk),zesf(ii,jj,kk)
     &     ,xvec(nxmax),esumr(nxmax),rhoav(ii),rhoav2(ii),logrhoav(ii),logrhoav2(ii),vlogrhoav(ii)
     &     ,vav(ii),vav2(ii),fdfac_ld(ii),r_ld(ii)
     &     ,yold1(ii),yold2(ii),yold3(ii),yold4(ii),yold5(ii),yold6(ii),yold7(ii)
      real adump(ntotal)
      integer ipa(ig,ig),jpa(ig,ig),nx
C
      common /zones/ zrho, zp, zux, zuy, zuz, zxa, zdx, zya, zdy, zza
     &              ,zdz, zgr, zgs, xmlag, zso, ztauc, zgssf, zesf
     &              ,imax, jmax, kmax, ipa, jpa, nx, xvec, esumr
     &              ,rhoav,rhoav2,vav,vav2,fdfac_ld,r_ld,logrhoav,logrhoav2,vlogrhoav 
     &              ,yold1,yold2,yold3,yold4,yold5,yold6,yold7				  
C
      equivalence (adump(1), zrho(1,1,1))



